#include "MainWindow.h"
#include <QtGui/QApplication>
#include "DefaultStyle.h"
#include "Share.h"
#include <QtGui/QIcon>
#include "globals.h"
#include <getopt.h>

static void
usage(void)
{
   printf("usage: %s [options...]\n", program);
   printf("\n");
   printf("NUU, Network Update Utility, is a graphical user interface which"
          " allows a user\n");
   printf("to make modifications to the installed rpm packages on a system.\n");
   printf("\n");
   printf("options:\n");
   printf("\n");
   printf("   -h, -H, --help           display this help information\n");
   printf("   -m m,   --mode=m         start with the package table mode,"
                                       " m:\n");
   printf("                               filter, search\n");
   printf("   -e r,   --regexp=r       start with the package table regexp,"
                                       " r\n");
   printf("   -s s,   --state=s        start with the package table state,"
                                       " s:\n");
   printf("                               updatable, installable, removable,"
                                       " all\n");
   printf("   -d,     --duplicates     start with the package table omitting"
                                       " duplicates\n");
   printf("   -g,     --groups         start with the package table showing"
                                       " groups\n");
   printf("   -r,     --repo-editor    start by bringing up the repository"
                                       " configuration\n");
   printf("                            editor\n");
   printf("   -l,     --log            start with the log window displayed\n");
   printf("           --disablerepo=r  disable the specified repository, r\n");
   printf("           --enablerepo=r   enable the specified repository, r\n");
   printf("\n");
   exit(1);
}

int main(int    argc, 
         char*  argv[])
{
   DefaultStyle  defaultStyle(argc, argv);
   QApplication  app(argc, argv);
   defaultStyle.set(app, "plastique");

   app.setWindowIcon(QIcon(Share::absolute("pixmaps/nuu.icon.png")));

   application = &app;
   program     = basename(argv[0]);

   MainWindow*  mw = new MainWindow(NULL);

#define DISABLEREPO  256
#define ENABLEREPO   257

   {
      static const char           shortopts[] = "+hHm:e:s:grl";
      static const struct option  longopts[]  = {
         { "help",        no_argument,       NULL, 'h'         },
         { "mode",        required_argument, NULL, 'm'         },
         { "regexp",      required_argument, NULL, 'e'         },
         { "state",       required_argument, NULL, 's'         },
         { "duplicates",  no_argument,       NULL, 'd'         },
         { "groups",      no_argument,       NULL, 'g'         },
         { "repo-editor", no_argument,       NULL, 'r'         },
         { "log",         no_argument,       NULL, 'l'         },
         { "disablerepo", required_argument, NULL, DISABLEREPO },
         { "enablerepo",  required_argument, NULL, ENABLEREPO  },
         { NULL,          no_argument,       NULL, '\0'        }
      };

      opterr = 0;

      for (;;) {
         int  longindex;
         int  opt = getopt_long(argc, argv, shortopts, longopts, &longindex);

         if (opt == EOF) break;

         switch (opt) {
         case 'h':
         case 'H':
            usage();
            /*NOTREACHED*/
         case 'm':
            {
               int  optlen = strlen(optarg);
               PackageTable::Mode  mode;
               if (strncmp(optarg, "filter", optlen) == 0) {
                  mode = PackageTable::FILTER;
               } else if (strncmp(optarg, "search", optlen) == 0) {
                  mode = PackageTable::SEARCH;
               } else {
                  fprintf(stderr,
                          "%s: unrecogized argument to -m/--mode option: %s\n",
                          program, optarg);
                  usage();
                  /*NOTREACHED*/
               }
               mw->setMode(mode);
            }
            break;
         case 'e':
            mw->setRegexp(optarg);
            break;
         case 's':
            {
               int  optlen = strlen(optarg);
               PackageTable::StateFilter  sf;
               if (strncmp(optarg, "updatable", optlen) == 0) {
                  sf = PackageTable::UPDATABLE;
               } else if (strncmp(optarg, "installable", optlen) == 0) {
                  sf = PackageTable::INSTALLABLE;
               } else if (strncmp(optarg, "removable", optlen) == 0) {
                  sf = PackageTable::REMOVABLE;
               } else if (strncmp(optarg, "all", optlen) == 0) {
                  sf = PackageTable::ALL;
               } else {
                  fprintf(stderr,
                          "%s: unrecogized argument to -s/--state option: %s\n",
                          program, optarg);
                  usage();
                  /*NOTREACHED*/
               }
               mw->setStateFilter(sf);
            }
            break;
         case 'd':
            mw->setDups(false);
            break;
         case 'g':
            mw->setGroups(true);
            break;
         case 'r':
            mw->optRepoEditor = true;
            break;
         case 'l':
            mw->optLog = true;
            break;
         case DISABLEREPO:
            mw->yumReposEnabled.append(YumRepoEnabled(false, optarg));
            break;
         case ENABLEREPO:
            mw->yumReposEnabled.append(YumRepoEnabled(true, optarg));
            break;
         case '?':
         default:
            if (optopt == '\0') {
               fprintf(stderr,
                       "%s: unrecogized option: %s\n", program, argv[optind-1]);
            } else {
               fprintf(stderr, 
                       "%s: unrecognized option letter: %c\n", program, optopt);
            }
            usage();
            /*NOTREACHED*/
         }
      }
   }

   if (optind < argc) {
      fprintf(stderr, "%s: unrecognized argument: %s\n", program, argv[optind]);
      usage();
      /*NOTREACHED*/
   }

   mw->show();

   return app.exec();
}
